@@ -27,7 +27,7 @@ The Document implementation keeps the document in a (unicode) text string,
 but you can inherit from the DocumentBase class to support other 
 representations of the text content.
 
-Modifying is done inside a context (the with statement), e.g.:
+Modifying is preferably done inside a context (the with statement), e.g.:
 
 .. code-block:: python
 
@@ -39,6 +39,10 @@ Modifying is done inside a context (the with statement), e.g.:
 Changes are applied when the context is exited, also the modified part of the
 document is re-tokenized. Changes may not overlap.
 
+You may modify the document outside a context, in which case the document is
+re-tokenized immediately. This is much slower however when performing multiple
+changes after each other.
+
 The tokens(block) method returns a tuple of tokens for the specified block. 
 Depending on the implementation, a block describes a line in the LilyPond 
 source document. It is not expected to have any methods, except that the 
@@ -221,11 +225,7 @@ class DocumentBase(object):
             self._changes.clear()
         elif self._writing == 1:
             if self._changes:
-                self._changes_list = [(start, end, text)
-                    for start, items in sorted(self._changes.items(), reverse=True)
-                    for end, text in reversed(sorted(items,
-                        key=lambda i: (i[0] is None, i[0])))]
-                self._changes.clear()
+                self._sort_changes()
                 self.update_cursors()
                 self.apply_changes()
                 del self._changes_list
@@ -252,6 +252,14 @@ class DocumentBase(object):
                 raise ValueError("overlapping edit: {0}-{1}: {2}".format(start, end, text))
             pos = start
     
+    def _sort_changes(self):
+        """Sort all the changes and put them in the _changes_list."""
+        self._changes_list = [(start, end, text)
+            for start, items in sorted(self._changes.items(), reverse=True)
+            for end, text in reversed(sorted(items,
+                key=lambda i: (i[0] is None, i[0])))]
+        self._changes.clear()
+    
     def update_cursors(self):
         """Updates the position of the registered Cursor instances."""
         for start, end, text in self._changes_list:
@@ -326,6 +334,12 @@ class DocumentBase(object):
         text = text.replace('\r', '')
         if text or start != end:
             self._changes[start].append((end, text))
+            # when a change is made outside context manager, apply immediately
+            if self._writing == 0:
+                self._sort_changes()
+                self.update_cursors()
+                self.apply_changes()
+                del self._changes_list
 
     def __delitem__(self, key):
         """Remove the range of text."""
@@ -832,14 +846,20 @@ class Source(object):
         """Initialize the iterator.
         
         cursor is a Cursor instance, describing a Document and a selected range
+        state is, if given, a ly.lex.State instance or True (in which case the
+            state is taken from the document).
+        
+        The following keyword arguments can be used:
+        
         partial is either OUTSIDE, PARTIAL, or INSIDE:
             OUTSIDE: tokens that touch the selected range are also yielded
             PARTIAL: tokens that overlap the start or end positions are yielded
             INSIDE:  (default) yield only tokens fully contained in the range
         The partial argument only makes sense if start or end are specified.
         
-        If tokens_with_position is True, uses the tokens_with_position() 
-        method to get the tokens, else (by default), the tokens() method is 
+        If tokens_with_position is True, uses the
+        document.tokens_with_position() method to get the tokens from the
+        cursor's document, else (by default), the document.tokens() method is
         used.
         
         """
