@@ -89,13 +89,12 @@ class CreateMusicXML():
             group_symbol = etree.SubElement(partgroup, "group-symbol")
             group_symbol.text = symbol
 
-    def create_part(self, name=False, abbr=False, midi=False):
+    def create_part(self, name="unnamed", abbr=False, midi=False):
         """Create a new part """
         strnr = str(self.part_count)
         part = etree.SubElement(self.partlist, "score-part", id="P"+strnr)
-        if name:
-            partname = etree.SubElement(part, "part-name")
-            partname.text = name
+        partname = etree.SubElement(part, "part-name")
+        partname.text = name
         if abbr:
             partabbr = etree.SubElement(part, "part-abbreviation")
             partabbr.text = abbr
@@ -134,8 +133,8 @@ class CreateMusicXML():
         self.add_pitch(step, alter, octave)
         if not grace[0]:
             self.add_div_duration(divdur)
-        self.add_duration_type(durtype)
         self.add_voice(voice)
+        self.add_duration_type(durtype)
         if dot:
             for i in range(dot):
                 self.add_dot()
@@ -164,18 +163,27 @@ class CreateMusicXML():
             for i in range(dot):
                 self.add_dot()
 
-    def tuplet_note(self, fraction, base_scaling, ttype, divs):
+    def tuplet_note(self, fraction, bs, ttype, nr, divs, atyp='', ntyp=''):
         """Convert current note to tuplet """
-        base = base_scaling[0]
-        scaling = base_scaling[1]
+        base = self.mult * bs[0]
+        scaling = bs[1]
         a = divs*4*fraction[1]
         b = (1/base)*fraction[0]
         duration = (a/b)*scaling
         self.change_div_duration(duration)
-        self.add_time_modify(fraction)
+        from fractions import Fraction
+        self.mult = Fraction(fraction[1], fraction[0])
+        timemod_node = self.get_time_modify()
+        if timemod_node:
+            self.adjust_time_modify(timemod_node, fraction)
+        else:
+            self.add_time_modify(fraction)
         if ttype:
             self.add_notations()
-            self.add_tuplet_type(ttype)
+            if atyp and ttype != "stop":
+                self.add_tuplet_type(nr, ttype, fraction[0], atyp, fraction[1], ntyp)
+            else:
+                self.add_tuplet_type(nr, ttype)
 
     def tie_note(self, tie_type):
         self.add_tie(tie_type)
@@ -330,6 +338,7 @@ class CreateMusicXML():
         """Create new duration """
         self.duration = etree.SubElement(self.current_note, "duration")
         self.duration.text = str(divdur)
+        self.mult = 1
 
     def change_div_duration(self, newdura):
         """Set new duration when tuplet """
@@ -376,9 +385,39 @@ class CreateMusicXML():
         norm_notes = etree.SubElement(timemod_node, "normal-notes")
         norm_notes.text = str(fraction[1])
 
-    def add_tuplet_type(self, ttype):
+    def get_time_modify(self):
+        """Check if time-modification node already exists."""
+        return self.current_note.find("time-modification")
+
+    def adjust_time_modify(self, timemod_node, fraction):
+        """Adjust existing time-modification node."""
+        actual_notes = timemod_node.find("actual-notes")
+        an = int(actual_notes.text) * fraction[0]
+        actual_notes.text = str(an)
+        norm_notes = timemod_node.find("normal-notes")
+        nn = int(norm_notes.text) * fraction[1]
+        norm_notes.text = str(nn)
+
+    def add_tuplet_type(self, nr, ttype, actnr=0, acttype='', normnr=0, normtype=''):
         """Create tuplet with type attribute """
-        etree.SubElement(self.current_notation, "tuplet", type=ttype)
+        tuplnode = etree.SubElement(self.current_notation, "tuplet",
+                                    {'number': str(nr), 'type': ttype })
+        if actnr:
+            actnode = etree.SubElement(tuplnode, "tuplet-actual")
+            atn = etree.SubElement(actnode, "tuplet-number")
+            atn.text = str(actnr)
+            att = etree.SubElement(actnode, "tuplet-type")
+            if not acttype:
+                acttype = self.current_note.find("type").text
+            att.text = acttype
+        if normnr:
+            normnode = etree.SubElement(tuplnode, "tuplet-normal")
+            ntn = etree.SubElement(normnode, "tuplet-number")
+            ntn.text = str(normnr)
+            ntt = etree.SubElement(normnode, "tuplet-type")
+            if not normtype:
+                normtype = self.current_note.find("type").text
+            ntt.text = normtype
 
     def add_slur(self, nr, sl_type):
         """Add slur. """
