@@ -148,6 +148,8 @@ class ModeShifter(Transposer):
             self.octave = -1
         elif self.steps < -3:
             self.octave = 1
+        else:
+            self.octave = 0
         self.alter = (self.scale[clp.note] + clp.alter -
                       self.scale[pitch.note] - pitch.alter)
         super(ModeShifter, self).transpose(pitch)
@@ -184,11 +186,10 @@ class ModalTransposer(object):
         # Look for an exact match: otherwise,
         # look for the letter name and save the accidental
         for i in range(len(self.notes)):
-            if pitch.note == self.notes[i] and \
-               pitch.alter == self.alter:
-                    fromScaleDeg = i
-                    accidental = 0
-                    break;
+            if pitch.note == self.notes[i] and pitch.alter == self.alter:
+                fromScaleDeg = i
+                accidental = 0
+                break
         else:
             fromScaleDeg = self.notes.index(pitch.note)
             accidental = pitch.alter - self.alter[fromScaleDeg]
@@ -199,14 +200,14 @@ class ModalTransposer(object):
         pitch.octave += toOctaveMod
 
 
-def transpose(cursor, transposer):
+def transpose(cursor, transposer, language="nederlands"):
     """Transpose pitches using the specified transposer."""
     start = cursor.start
     cursor.start = 0
     
     source = ly.document.Source(cursor, True, tokens_with_position=True)
 
-    pitches = ly.pitch.PitchIterator(source)
+    pitches = ly.pitch.PitchIterator(source, language)
     psource = pitches.pitches()
 
     class gen(object):
